@@ -20,7 +20,7 @@ import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.nio.ByteBuffer;
 import java.util.Iterator;
-import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.Lock;
@@ -32,7 +32,6 @@ import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.parser.Host;
 import org.apache.tomcat.util.log.UserDataHelper;
-import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.DispatchType;
 import org.apache.tomcat.util.net.SSLSupport;
@@ -49,9 +48,9 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     private static final StringManager sm = StringManager.getManager(AbstractProcessor.class);
 
     // Used to avoid useless B2C conversion on the host name.
-    protected char[] hostNameC = new char[0];
+    private char[] hostNameC = new char[0];
 
-    protected Adapter adapter;
+    protected final Adapter adapter;
     protected final AsyncStateMachine asyncStateMachine;
     private volatile long asyncTimeout = -1;
     /*
@@ -62,7 +61,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      * generation. This prevents the response mix-up.
      */
     private volatile long asyncTimeoutGeneration = 0;
-    protected final AbstractEndpoint<?, ?> endpoint;
     protected final Request request;
     protected final Response response;
     protected volatile SocketWrapperBase<?> socketWrapper = null;
@@ -76,13 +74,13 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
     protected final UserDataHelper userDataHelper;
 
-    public AbstractProcessor(AbstractEndpoint<?, ?> endpoint) {
-        this(endpoint, new Request(), new Response());
+    public AbstractProcessor(Adapter adapter) {
+        this(adapter, new Request(), new Response());
     }
 
 
-    protected AbstractProcessor(AbstractEndpoint<?, ?> endpoint, Request coyoteRequest, Response coyoteResponse) {
-        this.endpoint = endpoint;
+    protected AbstractProcessor(Adapter adapter, Request coyoteRequest, Response coyoteResponse) {
+        this.adapter = adapter;
         asyncStateMachine = new AsyncStateMachine(this);
         request = coyoteRequest;
         response = coyoteResponse;
@@ -100,13 +98,14 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      * @param errorState The error status details
      * @param t          The error which occurred
      */
+    @SuppressWarnings("deprecation")
     protected void setErrorState(ErrorState errorState, Throwable t) {
         if (getLog().isDebugEnabled()) {
             getLog().debug(sm.getString("abstractProcessor.setErrorState", errorState), t);
         }
         // Use the return value to avoid processing more than one async error
         // in a single async cycle.
-        boolean setError = response.setError();
+        response.setError();
         boolean blockIo = this.errorState.isIoAllowed() && !errorState.isIoAllowed();
         this.errorState = this.errorState.getMostSevere(errorState);
         // Don't change the status code for IOException since that is almost
@@ -118,7 +117,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
         if (t != null) {
             request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
         }
-        if (blockIo && isAsync() && setError) {
+        if (blockIo && isAsync()) {
             if (asyncStateMachine.asyncError()) {
                 processSocketEvent(SocketEvent.ERROR, true);
             }
@@ -137,16 +136,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
-    /**
-     * Set the associated adapter.
-     *
-     * @param adapter the new adapter
-     */
-    public void setAdapter(Adapter adapter) {
-        this.adapter = adapter;
-    }
-
-
     /**
      * Get the associated adapter.
      *
@@ -182,10 +171,17 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
 
     /**
-     * @return the Executor used by the underlying endpoint.
+     * Provides a mechanism to trigger processing on a container thread.
+     *
+     * @param runnable The task representing the processing that needs to take place on a container thread
      */
-    protected Executor getExecutor() {
-        return endpoint.getExecutor();
+    protected void execute(Runnable runnable) {
+        SocketWrapperBase<?> socketWrapper = this.socketWrapper;
+        if (socketWrapper == null) {
+            throw new RejectedExecutionException(sm.getString("abstractProcessor.noExecute"));
+        } else {
+            socketWrapper.execute(runnable);
+        }
     }
 
 
@@ -212,7 +208,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                 }
             } catch (IOException ioe) {
                 if (getLog().isDebugEnabled()) {
-                    getLog().debug("Unable to write async data.", ioe);
+                    getLog().debug(sm.getString("abstractProcessor.asyncFail"), ioe);
                 }
                 status = SocketEvent.ERROR;
                 request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, ioe);
@@ -242,7 +238,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
         RequestInfo rp = request.getRequestProcessor();
         try {
-            rp.setStage(org.apache.coyote.Constants.STAGE_SERVICE);
+            rp.setStage(Constants.STAGE_SERVICE);
             if (!getAdapter().asyncDispatch(request, response, status)) {
                 setErrorState(ErrorState.CLOSE_NOW, null);
             }
@@ -254,7 +250,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             getLog().error(sm.getString("http11processor.request.process"), t);
         }
 
-        rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
+        rp.setStage(Constants.STAGE_ENDED);
 
         SocketState state;
 
@@ -268,8 +264,8 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             state = dispatchEndRequest();
         }
 
-        if (getLog().isDebugEnabled()) {
-            getLog().debug("Socket: [" + socketWrapper + "], Status in: [" + status + "], State out: [" + state + "]");
+        if (getLog().isTraceEnabled()) {
+            getLog().trace("Socket: [" + socketWrapper + "], Status in: [" + status + "], State out: [" + state + "]");
         }
 
         return state;
@@ -619,6 +615,18 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                 break;
             }
 
+            // Servlet 4.0 Trailers
+            case IS_TRAILER_FIELDS_READY: {
+                AtomicBoolean result = (AtomicBoolean) param;
+                result.set(isTrailerFieldsReady());
+                break;
+            }
+            case IS_TRAILER_FIELDS_SUPPORTED: {
+                AtomicBoolean result = (AtomicBoolean) param;
+                result.set(isTrailerFieldsSupported());
+                break;
+            }
+
             // Identifiers associated with multiplexing protocols like HTTP/2
             case CONNECTION_ID: {
                 @SuppressWarnings("unchecked")
@@ -712,9 +720,17 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
+    /**
+     * When committing the response, we have to validate the set of headers, as well as setup the response filters.
+     * @throws IOException IO exception during commit
+     */
     protected abstract void prepareResponse() throws IOException;
 
 
+    /**
+     * Finish the current response.
+     * @throws IOException IO exception during the write
+     */
     protected abstract void finishResponse() throws IOException;
 
 
@@ -727,21 +743,46 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
+    /**
+     * Process acknowledgment of the request.
+     * @param continueResponseTiming specifies when an acknowledgment should be sent
+     */
     protected abstract void ack(ContinueResponseTiming continueResponseTiming);
 
 
+    /**
+     * Callback to write data from the buffer.
+     * @throws IOException IO exception during the write
+     */
     protected abstract void flush() throws IOException;
 
 
+    /**
+     * Queries if bytes are available in buffers.
+     * @param doRead {@code true} to perform a read when no bytes are availble
+     * @return the amount of bytes that are known to be available
+     */
     protected abstract int available(boolean doRead);
 
 
+    /**
+     * Set the specified byte chunk as the request body that will be read. This allows saving and
+     * processing requests.
+     * @param body the byte chunk containing all the request bytes
+     */
     protected abstract void setRequestBody(ByteChunk body);
 
 
+    /**
+     * The response is finished and no additional bytes need to be sent to the client.
+     */
     protected abstract void setSwallowResponse();
 
 
+    /**
+     * Swallowing bytes is required for pipelining requests, so this allows to avoid doing extra operations
+     * in case an error occurs and the connection is to be closed instead.
+     */
     protected abstract void disableSwallowRequest();
 
 
@@ -844,12 +885,22 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
+    /**
+     * @return {@code true} if it is known that the request body has been fully read
+     */
     protected abstract boolean isRequestBodyFullyRead();
 
 
+    /**
+     * When using non blocking IO, register to get a callback when polling determines that bytes
+     * are available for reading.
+     */
     protected abstract void registerReadInterest();
 
 
+    /**
+     * @return {@code true} if bytes can be written without blocking
+     */
     protected abstract boolean isReadyForWrite();
 
 
@@ -950,6 +1001,19 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
+    protected abstract boolean isTrailerFieldsReady();
+
+
+    /**
+     * Protocols that support trailer fields should override this method and return {@code true}.
+     *
+     * @return {@code true} if trailer fields are supported by this processor, otherwise {@code false}.
+     */
+    protected boolean isTrailerFieldsSupported() {
+        return false;
+    }
+
+
     /**
      * Protocols that support multiplexing (e.g. HTTP/2) should override this method and return the appropriate ID.
      *
@@ -992,6 +1056,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     protected abstract SocketState dispatchEndRequest() throws IOException;
 
 
+    @SuppressWarnings("deprecation")
     @Override
     protected final void logAccess(SocketWrapperBase<?> socketWrapper) throws IOException {
         // Set the socket wrapper so the access log can read the socket related
