@@ -22,15 +22,16 @@ import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
 import java.nio.charset.Charset;
 import java.util.Locale;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
+import java.util.function.Supplier;
 
 import javax.servlet.WriteListener;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.B2CConverter;
-import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.MediaType;
@@ -80,6 +81,8 @@ public final class Response {
     final MimeHeaders headers = new MimeHeaders();
 
 
+    private Supplier<Map<String, String>> trailerFieldsSupplier = null;
+
     /**
      * Associated output buffer.
      */
@@ -275,7 +278,9 @@ public final class Response {
      * @param ex The exception that occurred
      */
     public void setErrorException(Exception ex) {
-        errorException = ex;
+        if (errorException == null) {
+            errorException = ex;
+        }
     }
 
 
@@ -298,7 +303,10 @@ public final class Response {
      * Set the error flag.
      *
      * @return <code>false</code> if the error flag was already set
+     *
+     * @deprecated This method will be changed to return void in Tomcat 11 onwards
      */
+    @Deprecated
     public boolean setError() {
         return errorState.compareAndSet(0, 1);
     }
@@ -324,6 +332,11 @@ public final class Response {
     }
 
 
+    public void resetError() {
+        errorState.set(0);
+    }
+
+
     // -------------------- Methods --------------------
 
     public void reset() throws IllegalStateException {
@@ -381,6 +394,22 @@ public final class Response {
     }
 
 
+    public void setTrailerFields(Supplier<Map<String, String>> supplier) {
+        AtomicBoolean trailerFieldsSupported = new AtomicBoolean(false);
+        action(ActionCode.IS_TRAILER_FIELDS_SUPPORTED, trailerFieldsSupported);
+        if (!trailerFieldsSupported.get()) {
+            throw new IllegalStateException(sm.getString("response.noTrailers.notSupported"));
+        }
+
+        this.trailerFieldsSupplier = supplier;
+    }
+
+
+    public Supplier<Map<String, String>> getTrailerFields() {
+        return trailerFieldsSupplier;
+    }
+
+
     /**
      * Set internal fields for special header names. Called from set/addHeader. Return true if the header is special, no
      * need to set the header.
@@ -459,8 +488,10 @@ public final class Response {
      * output using getWriter().
      *
      * @param characterEncoding The name of character encoding.
+     *
+     * @throws UnsupportedEncodingException If the specified name is not recognised
      */
-    public void setCharacterEncoding(String characterEncoding) {
+    public void setCharacterEncoding(String characterEncoding) throws UnsupportedEncodingException {
         if (isCommitted()) {
             return;
         }
@@ -471,11 +502,7 @@ public final class Response {
         }
 
         this.characterEncoding = characterEncoding;
-        try {
-            this.charset = B2CConverter.getCharset(characterEncoding);
-        } catch (UnsupportedEncodingException e) {
-            throw new IllegalArgumentException(e);
-        }
+        this.charset = B2CConverter.getCharset(characterEncoding);
     }
 
 
@@ -574,22 +601,6 @@ public final class Response {
     }
 
 
-    /**
-     * Write a chunk of bytes.
-     *
-     * @param chunk The bytes to write
-     *
-     * @throws IOException If an I/O error occurs during the write
-     *
-     * @deprecated Unused. Will be removed in Tomcat 9. Use {@link #doWrite(ByteBuffer)}
-     */
-    @Deprecated
-    public void doWrite(ByteChunk chunk) throws IOException {
-        outputBuffer.doWrite(chunk);
-        contentWritten += chunk.getLength();
-    }
-
-
     /**
      * Write a chunk of bytes.
      *
@@ -618,8 +629,9 @@ public final class Response {
         committed = false;
         commitTime = -1;
         errorException = null;
-        errorState.set(0);
+        resetError();
         headers.recycle();
+        trailerFieldsSupplier = null;
         // Servlet 3.1 non-blocking write listener
         listener = null;
         synchronized (nonBlockingStateLock) {
