@@ -22,11 +22,11 @@ import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 
+import org.apache.coyote.CloseNowException;
 import org.apache.coyote.InputBuffer;
 import org.apache.coyote.Request;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.HeaderUtil;
 import org.apache.tomcat.util.http.MimeHeaders;
@@ -236,21 +236,6 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
 
     // ---------------------------------------------------- InputBuffer Methods
 
-    /**
-     * @deprecated Unused. Will be removed in Tomcat 9. Use {@link #doRead(ApplicationBufferHandler)}
-     */
-    @Deprecated
-    @Override
-    public int doRead(ByteChunk chunk) throws IOException {
-
-        if (lastActiveFilter == -1) {
-            return inputStreamInputBuffer.doRead(chunk);
-        } else {
-            return activeFilters[lastActiveFilter].doRead(chunk);
-        }
-
-    }
-
     @Override
     public int doRead(ApplicationBufferHandler handler) throws IOException {
         if (lastActiveFilter == -1) {
@@ -342,7 +327,7 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
      *
      * @return true if data is properly fed; false if no data is available immediately and thread should be freed
      */
-    boolean parseRequestLine(boolean keptAlive) throws IOException {
+    boolean parseRequestLine(boolean keptAlive, int connectionTimeout, int keepAliveTimeout) throws IOException {
 
         // check state
         if (!parsingRequestLine) {
@@ -358,7 +343,7 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
                     if (keptAlive) {
                         // Haven't read any request data yet so use the keep-alive
                         // timeout.
-                        wrapper.setReadTimeout(wrapper.getEndpoint().getKeepAliveTimeout());
+                        wrapper.setReadTimeout(keepAliveTimeout);
                     }
                     if (!fill(false)) {
                         // A read is pending, so no longer in initial state
@@ -367,7 +352,7 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
                     }
                     // At least one byte of the request has been received.
                     // Switch to the socket timeout.
-                    wrapper.setReadTimeout(wrapper.getEndpoint().getConnectionTimeout());
+                    wrapper.setReadTimeout(connectionTimeout);
                 }
                 if (!keptAlive && byteBuffer.position() == 0 && byteBuffer.limit() >= CLIENT_PREFACE_START.length) {
                     boolean prefaceMatch = true;
@@ -594,6 +579,8 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
 
     /**
      * Parse the HTTP headers.
+     *
+     * @throws IOException an underlying I/O error occurred
      */
     boolean parseHeaders() throws IOException {
         if (!parsingHeader) {
@@ -671,6 +658,7 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
     /**
      * Available bytes in the buffers for the current request. Note that when requests are pipelined, the data in
      * byteBuffer may relate to the next request rather than this one.
+     * @return the amount of bytes available, 0 if none, and 1 if there was an IO error to trigger a read
      */
     int available(boolean read) {
         int available;
@@ -713,6 +701,7 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
     /**
      * Has all of the request body been read? There are subtle differences between this and available() &gt; 0 primarily
      * because of having to handle faking non-blocking reads with the blocking IO connector.
+     * @return {@code true} if the request has been fully read
      */
     boolean isFinished() {
         // The active filters have the definitive information on whether or not
@@ -737,6 +726,16 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
     }
 
 
+    boolean isChunking() {
+        for (int i = 0; i < lastActiveFilter; i++) {
+            if (activeFilters[i] == filterLibrary[Constants.CHUNKED_FILTER]) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+
     void init(SocketWrapperBase<?> socketWrapper) {
 
         wrapper = socketWrapper;
@@ -785,7 +784,12 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
                 byteBuffer.position(byteBuffer.limit());
             }
             byteBuffer.limit(byteBuffer.capacity());
-            nRead = wrapper.read(block, byteBuffer);
+            SocketWrapperBase<?> socketWrapper = this.wrapper;
+            if (socketWrapper != null) {
+                nRead = socketWrapper.read(block, byteBuffer);
+            } else {
+                throw new CloseNowException(sm.getString("iib.eof.error"));
+            }
         } finally {
             // Ensure that the buffer limit and position are returned to a
             // consistent "ready for read" state if an error occurs during in
@@ -1182,28 +1186,6 @@ public class Http11InputBuffer implements InputBuffer, ApplicationBufferHandler
      */
     private class SocketInputBuffer implements InputBuffer {
 
-        /**
-         * @deprecated Unused. Will be removed in Tomcat 9. Use {@link #doRead(ApplicationBufferHandler)}
-         */
-        @Deprecated
-        @Override
-        public int doRead(ByteChunk chunk) throws IOException {
-
-            if (byteBuffer.position() >= byteBuffer.limit()) {
-                // The application is reading the HTTP request body which is
-                // always a blocking operation.
-                if (!fill(true)) {
-                    return -1;
-                }
-            }
-
-            int length = byteBuffer.remaining();
-            chunk.setBytes(byteBuffer.array(), byteBuffer.position(), length);
-            byteBuffer.position(byteBuffer.limit());
-
-            return length;
-        }
-
         @Override
         public int doRead(ApplicationBufferHandler handler) throws IOException {
 
